@@ -1,12 +1,10 @@
 # -*- test-case-name: twisted.test.test_defer -*-
 #
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
-
-"""Support for results that aren't immediately available.
-
-API Stability: stable
+"""
+Support for results that aren't immediately available.
 
 Maintainer: U{Glyph Lefkowitz<mailto:glyph@twistedmatrix.com>}
 """
@@ -16,7 +14,7 @@ import traceback
 import warnings
 
 # Twisted imports
-from twisted.python import log, failure
+from twisted.python import log, failure, lockfile
 from twisted.python.util import unsignedID, mergeFunctionMetadata
 
 class AlreadyCalledError(Exception):
@@ -155,6 +153,11 @@ class Deferred:
     timeoutCall = None
     _debugInfo = None
 
+    # Are we currently running a user-installed callback?  Meant to prevent
+    # recursive running of callbacks when a reentrant call to add a callback is
+    # used.
+    _runningCallbacks = False
+
     # Keep this class attribute for now, for compatibility with code that
     # sets it directly.
     debug = False
@@ -304,26 +307,31 @@ class Deferred:
         self._runCallbacks()
 
     def _runCallbacks(self):
+        if self._runningCallbacks:
+            # Don't recursively run callbacks
+            return
         if not self.paused:
-            cb = self.callbacks
-            self.callbacks = []
-            while cb:
-                item = cb.pop(0)
+            while self.callbacks:
+                item = self.callbacks.pop(0)
                 callback, args, kw = item[
                     isinstance(self.result, failure.Failure)]
                 args = args or ()
                 kw = kw or {}
                 try:
-                    self.result = callback(self.result, *args, **kw)
+                    self._runningCallbacks = True
+                    try:
+                        self.result = callback(self.result, *args, **kw)
+                    finally:
+                        self._runningCallbacks = False
                     if isinstance(self.result, Deferred):
-                        self.callbacks = cb
-
                         # note: this will cause _runCallbacks to be called
-                        # "recursively" sometimes... this shouldn't cause any
-                        # problems, since all the state has been set back to
-                        # the way it's supposed to be, but it is useful to know
-                        # in case something goes wrong.  deferreds really ought
-                        # not to return themselves from their callbacks.
+                        # recursively if self.result already has a result.
+                        # This shouldn't cause any problems, since there is no
+                        # relevant state in this stack frame at this point.
+                        # The recursive call will continue to process
+                        # self.callbacks until it is empty, then return here,
+                        # where there is no more work to be done, so this call
+                        # will return as well.
                         self.pause()
                         self.result.addBoth(self._continue)
                         break
@@ -615,8 +623,6 @@ def _deferGenerator(g, deferred):
 
 def deferredGenerator(f):
     """
-    API Stability: stable
-
     Maintainer: U{Christopher Armstrong<mailto:radix@twistedmatrix.com>}
 
     deferredGenerator and waitForDeferred help you write Deferred-using code
@@ -721,7 +727,7 @@ def _inlineCallbacks(result, g, deferred):
         try:
             # Send the last result back as the result of the yield expression.
             if isinstance(result, failure.Failure):
-                result = g.throw(result.type, result.value, result.tb)
+                result = result.throwExceptionIntoGenerator(g)
             else:
                 result = g.send(result)
         except StopIteration:
@@ -767,8 +773,6 @@ def _inlineCallbacks(result, g, deferred):
 
 def inlineCallbacks(f):
     """
-    API Stability: semi-stable
-
     Maintainer: U{Christopher Armstrong<mailto:radix@twistedmatrix.com>}
 
     WARNING: this function will not work in Python 2.4 and earlier!
@@ -777,7 +781,7 @@ def inlineCallbacks(f):
     regular sequential function. This function uses features of Python 2.5
     generators.  If you need to be compatible with Python 2.4 or before, use
     the L{deferredGenerator} function instead, which accomplishes the same
-    thing, but with somewhat more boilerplate.
+    thing, but with somewhat more boilerplate.  For example::
 
         def thingummy():
             thing = yield makeSomeRequestResultingInDeferred()
@@ -797,7 +801,7 @@ def inlineCallbacks(f):
     will cause the Deferred to have a result of None.
 
     The Deferred returned from your deferred generator may errback if your
-    generator raised an exception.
+    generator raised an exception::
 
         def thingummy():
             thing = yield makeSomeRequestResultingInDeferred()
@@ -856,9 +860,8 @@ class _ConcurrencyPrimitive(object):
 
 
 class DeferredLock(_ConcurrencyPrimitive):
-    """A lock for event driven systems.
-
-    API stability: Unstable
+    """
+    A lock for event driven systems.
 
     @ivar locked: True when this Lock has been acquired, false at all
     other times.  Do not change this value, but it is useful to
@@ -895,9 +898,8 @@ class DeferredLock(_ConcurrencyPrimitive):
             d.callback(self)
 
 class DeferredSemaphore(_ConcurrencyPrimitive):
-    """A semaphore for event driven systems.
-
-    API stability: Unstable
+    """
+    A semaphore for event driven systems.
     """
 
     def __init__(self, tokens):
@@ -941,9 +943,8 @@ class QueueUnderflow(Exception):
 
 
 class DeferredQueue(object):
-    """An event driven queue.
-
-    API stability: Unstable
+    """
+    An event driven queue.
 
     Objects may be added as usual to this queue.  When an attempt is
     made to retrieve an object when the queue is empty, a Deferred is
@@ -995,9 +996,107 @@ class DeferredQueue(object):
             raise QueueUnderflow()
 
 
+class AlreadyTryingToLockError(Exception):
+    """
+    Raised when DeferredFilesystemLock.deferUntilLocked is called twice on a
+    single DeferredFilesystemLock.
+    """
+
+
+class DeferredFilesystemLock(lockfile.FilesystemLock):
+    """
+    A FilesystemLock that allows for a deferred to be fired when the lock is
+    acquired.
+
+    @ivar _scheduler: The object in charge of scheduling retries. In this
+        implementation this is parameterized for testing.
+
+    @ivar _interval: The retry interval for an L{IReactorTime} based scheduler.
+
+    @ivar _tryLockCall: A L{DelayedCall} based on _interval that will managex
+        the next retry for aquiring the lock.
+
+    @ivar _timeoutCall: A L{DelayedCall} based on deferUntilLocked's timeout
+        argument.  This is in charge of timing out our attempt to acquire the
+        lock.
+    """
+    _interval = 1
+    _tryLockCall = None
+    _timeoutCall = None
+
+    def __init__(self, name, scheduler=None):
+        """
+        @param name: The name of the lock to acquire
+        @param scheduler: An object which provides L{IReactorTime}
+        """
+        lockfile.FilesystemLock.__init__(self, name)
+
+        if scheduler is None:
+            from twisted.internet import reactor
+            scheduler = reactor
+
+        self._scheduler = scheduler
+
+    def deferUntilLocked(self, timeout=None):
+        """
+        Wait until we acquire this lock.  This method is not safe for
+        concurrent use.
+
+        @type timeout: C{float} or C{int}
+        @param timeout: the number of seconds after which to time out if the
+            lock has not been acquired.
+
+        @return: a deferred which will callback when the lock is acquired, or
+            errback with a L{TimeoutError} after timing out or an
+            L{AlreadyTryingToLockError} if the L{deferUntilLocked} has already
+            been called and not successfully locked the file.
+        """
+        if self._tryLockCall is not None:
+            return fail(
+                AlreadyTryingToLockError(
+                    "deferUntilLocked isn't safe for concurrent use."))
+
+        d = Deferred()
+
+        def _cancelLock():
+            self._tryLockCall.cancel()
+            self._tryLockCall = None
+            self._timeoutCall = None
+
+            if self.lock():
+                d.callback(None)
+            else:
+                d.errback(failure.Failure(
+                        TimeoutError("Timed out aquiring lock: %s after %fs" % (
+                                self.name,
+                                timeout))))
+
+        def _tryLock():
+            if self.lock():
+                if self._timeoutCall is not None:
+                    self._timeoutCall.cancel()
+                    self._timeoutCall = None
+
+                self._tryLockCall = None
+
+                d.callback(None)
+            else:
+                if timeout is not None and self._timeoutCall is None:
+                    self._timeoutCall = self._scheduler.callLater(
+                        timeout, _cancelLock)
+
+                self._tryLockCall = self._scheduler.callLater(
+                    self._interval, _tryLock)
+
+        _tryLock()
+
+        return d
+
+
 __all__ = ["Deferred", "DeferredList", "succeed", "fail", "FAILURE", "SUCCESS",
            "AlreadyCalledError", "TimeoutError", "gatherResults",
            "maybeDeferred",
            "waitForDeferred", "deferredGenerator", "inlineCallbacks",
            "DeferredLock", "DeferredSemaphore", "DeferredQueue",
+           "DeferredFilesystemLock", "AlreadyTryingToLockError",
           ]
