@@ -1,11 +1,9 @@
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
-"""Interface documentation.
-
-API Stability: stable, other than IReactorUDP (semi-stable) and
-IReactorMulticast (unstable).
+"""
+Interface documentation.
 
 Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
@@ -366,9 +364,12 @@ class IReactorProcess(Interface):
 
     def spawnProcess(processProtocol, executable, args=(), env={}, path=None,
                      uid=None, gid=None, usePTY=0, childFDs=None):
-        """Spawn a process, with a process protocol.
+        """
+        Spawn a process, with a process protocol.
 
-        @param processProtocol: a L{twisted.internet.protocol.ProcessProtocol} instance
+        @type processProtocol: L{IProcessProtocol} provider
+        @param processProtocol: An object which will be notified of all
+            events related to the created process.
 
         @param executable: the file name to spawn - the full path should be
                            used.
@@ -407,7 +408,7 @@ class IReactorProcess(Interface):
                          attached to the child at that file descriptor: the
                          child will be able to write to that file descriptor
                          and the parent will receive read notification via the
-                         L{IProcessTransport.childDataReceived} callback.  This
+                         L{IProcessProtocol.childDataReceived} callback.  This
                          is useful for the child's stdout and stderr.
 
                          If it is the string 'w', similar setup to the previous
@@ -430,11 +431,21 @@ class IReactorProcess(Interface):
         """
 
 class IReactorTime(Interface):
-    """Time methods that a Reactor should implement.
+    """
+    Time methods that a Reactor should implement.
     """
 
+    def seconds():
+        """
+        Get the current time in seconds.
+
+        @return: A number-like object of some sort.
+        """
+
+
     def callLater(delay, callable, *args, **kw):
-        """Call a function later.
+        """
+        Call a function later.
 
         @type delay:  C{float}
         @param delay: the number of seconds to wait.
@@ -452,7 +463,8 @@ class IReactorTime(Interface):
         """
 
     def cancelCallLater(callID):
-        """This method is deprecated.
+        """
+        This method is deprecated.
 
         Cancel a call that would happen later.
 
@@ -463,7 +475,8 @@ class IReactorTime(Interface):
         """
 
     def getDelayedCalls():
-        """Retrieve all currently scheduled delayed calls.
+        """
+        Retrieve all currently scheduled delayed calls.
 
         @return: A tuple of all L{IDelayedCall} providers representing all
                  currently scheduled calls. This is everything that has been
@@ -472,20 +485,23 @@ class IReactorTime(Interface):
 
 
 class IDelayedCall(Interface):
-    """A scheduled call.
+    """
+    A scheduled call.
 
     There are probably other useful methods we can add to this interface;
     suggestions are welcome.
     """
 
     def getTime():
-        """Get time when delayed call will happen.
+        """
+        Get time when delayed call will happen.
 
         @return: time in seconds since epoch (a float).
         """
 
     def cancel():
-        """Cancel the scheduled call.
+        """
+        Cancel the scheduled call.
 
         @raises twisted.internet.error.AlreadyCalled: if the call has already
             happened.
@@ -494,7 +510,8 @@ class IDelayedCall(Interface):
         """
 
     def delay(secondsLater):
-        """Delay the scheduled call.
+        """
+        Delay the scheduled call.
 
         @param secondsLater: how many seconds from its current firing time to delay
 
@@ -505,7 +522,8 @@ class IDelayedCall(Interface):
         """
 
     def reset(secondsFromNow):
-        """Reset the scheduled call's timer.
+        """
+        Reset the scheduled call's timer.
 
         @param secondsFromNow: how many seconds from now it should fire,
             equivalent to C{.cancel()} and then doing another
@@ -640,6 +658,15 @@ class IReactorCore(Interface):
         """Removes a trigger added with addSystemEventTrigger.
 
         @param triggerID: a value returned from addSystemEventTrigger.
+
+        @raise KeyError: If there is no system event trigger for the given
+            C{triggerID}.
+
+        @raise ValueError: If there is no system event trigger for the given
+            C{triggerID}.
+
+        @raise TypeError: If there is no system event trigger for the given
+            C{triggerID}.
         """
 
     def callWhenRunning(callable, *args, **kw):
@@ -724,6 +751,27 @@ class IReactorFDSet(Interface):
         """
 
 
+    def getReaders():
+        """
+        Return the list of file descriptors currently monitored for input
+        events by the reactor.
+
+        @return: the list of file descriptors monitored for input events.
+        @rtype: C{list} of C{IReadDescriptor}
+        """
+
+
+    def getWriters():
+        """
+        Return the list file descriptors currently monitored for output events
+        by the reactor.
+
+        @return: the list of file descriptors monitored for output events.
+        @rtype: C{list} of C{IWriteDescriptor}
+        """
+
+
+
 class IListeningPort(Interface):
     """A listening port.
     """
@@ -750,8 +798,21 @@ class IListeningPort(Interface):
         """
 
 
-class IFileDescriptor(Interface):
-    """A file descriptor.
+class ILoggingContext(Interface):
+    """
+    Give context information that will be used to log events generated by
+    this item.
+    """
+    def logPrefix():
+        """
+        @return: Prefix used during log formatting to indicate context.
+        @rtype: C{str}
+        """
+
+
+class IFileDescriptor(ILoggingContext):
+    """
+    A file descriptor.
     """
 
     def fileno():
@@ -858,8 +919,6 @@ class IConsumer(Interface):
 
 class IFinishableConsumer(IConsumer):
     """A Consumer for producers that finish.
-
-    This interface is semi-stable.
     """
     def finish():
         """The producer has finished producing."""
@@ -886,8 +945,6 @@ class IPushProducer(IProducer):
     it has been paused. A paused push producer will resume producing
     after its resumeProducing() method is called.   For a push producer
     which is not pauseable, these functions may be noops.
-
-    This interface is semi-stable.
     """
 
     def pauseProducing():
@@ -907,8 +964,6 @@ class IPullProducer(IProducer):
     """
     A pull producer, also known as a non-streaming producer, is
     expected to produce data each time resumeProducing() is called.
-
-    This interface is semi-stable.
     """
 
     def resumeProducing():
@@ -964,6 +1019,57 @@ class IProtocol(Interface):
         """
 
 
+class IProcessProtocol(Interface):
+    """
+    Interface for process-related event handlers.
+    """
+
+    def makeConnection(process):
+        """
+        Called when the process has been created.
+
+        @type process: L{IProcessTransport} provider
+        @param process: An object representing the process which has been
+            created and associated with this protocol.
+        """
+
+
+    def childDataReceived(childFD, data):
+        """
+        Called when data arrives from the child process.
+
+        @type childFD: C{int}
+        @param childFD: The file descriptor from which the data was
+            received.
+
+        @type data: C{str}
+        @param data: The data read from the child's file descriptor.
+        """
+
+
+    def childConnectionLost(childFD):
+        """
+        Called when a file descriptor associated with the child process is
+        closed.
+
+        @type childFD: C{int}
+        @param childFD: The file descriptor which was closed.
+        """
+
+
+    def processEnded(reason):
+        """
+        Called when the child process exits.
+
+        @type reason: L{twisted.python.failure.Failure}
+        @param reason: A failure giving the reason the child process
+            terminated.  The type of exception for this failure is either
+            L{twisted.internet.error.ProcessDone} or
+            L{twisted.internet.error.ProcessTerminated}.
+        """
+
+
+
 class IHalfCloseableProtocol(Interface):
     """Implemented to indicate they want notification of half-closes.
 
