@@ -1,13 +1,11 @@
 # -*- test-case-name: twisted.test.test_internet -*-
-# $Id: default.py,v 1.90 2004/01/06 22:35:22 warner Exp $
 #
-# Copyright (c) 2001-2004 Twisted Matrix Laboratories.
+# Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
 
-"""Posix reactor base class
-
-API Stability: stable
+"""
+Posix reactor base class
 
 Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
@@ -27,7 +25,7 @@ from twisted.internet.interfaces import IHalfCloseableDescriptor
 from twisted.internet import error
 from twisted.internet import tcp, udp
 
-from twisted.python import log, threadable, failure, util
+from twisted.python import log, failure, util
 from twisted.persisted import styles
 from twisted.python.runtime import platformType, platform
 
@@ -134,6 +132,8 @@ class _UnixWaker(log.Logger, styles.Ephemeral):
     def wakeUp(self):
         """Write one byte to the pipe, and flush it.
         """
+        # We don't use fdesc.writeToFD since we need to distinguish
+        # between EINTR (try again) and EAGAIN (do nothing).
         if self.o is not None:
             try:
                 util.untilConcludes(os.write, self.o, 'x')
@@ -162,7 +162,8 @@ elif platformType == 'win32':
 
 
 class PosixReactorBase(ReactorBase):
-    """A basis for reactors that use file descriptors.
+    """
+    A basis for reactors that use file descriptors.
     """
     implements(IReactorArbitrary, IReactorTCP, IReactorUDP, IReactorMulticast)
 
@@ -172,11 +173,14 @@ class PosixReactorBase(ReactorBase):
             self.installWaker()
 
     def _handleSignals(self):
-        """Install the signal handlers for the Twisted event loop."""
+        """
+        Install the signal handlers for the Twisted event loop.
+        """
         try:
             import signal
         except ImportError:
-            log.msg("Warning: signal module unavailable -- not installing signal handlers.")
+            log.msg("Warning: signal module unavailable -- "
+                    "not installing signal handlers.")
             return
 
         if signal.getsignal(signal.SIGINT) == signal.default_int_handler:
@@ -192,7 +196,8 @@ class PosixReactorBase(ReactorBase):
             signal.signal(signal.SIGCHLD, self._handleSigchld)
 
     def _handleSigchld(self, signum, frame, _threadSupport=platform.supportsThreads()):
-        """Reap all processes on SIGCHLD.
+        """
+        Reap all processes on SIGCHLD.
 
         This gets called on SIGCHLD. We do no processing inside a signal
         handler, as the calls we make here could occur between any two
@@ -205,17 +210,15 @@ class PosixReactorBase(ReactorBase):
         else:
             self.callLater(0, process.reapAllProcesses)
 
-    def startRunning(self, installSignalHandlers=1):
-        # Just in case we're started on a different thread than
-        # we're made on
-        threadable.registerAsIOThread()
-
-        self.fireSystemEvent('startup')
+    def startRunning(self, installSignalHandlers=True):
+        """
+        Forward call to ReactorBase, install signal handlers if asked.
+        """
+        ReactorBase.startRunning(self)
         if installSignalHandlers:
             self._handleSignals()
-        self.running = 1
 
-    def run(self, installSignalHandlers=1):
+    def run(self, installSignalHandlers=True):
         self.startRunning(installSignalHandlers=installSignalHandlers)
         self.mainLoop()
 
@@ -239,7 +242,8 @@ class PosixReactorBase(ReactorBase):
         error.ConnectionDone: failure.Failure(error.ConnectionDone()),
         error.ConnectionLost: failure.Failure(error.ConnectionLost())
         }):
-        """Utility function for disconnecting a selectable.
+        """
+        Utility function for disconnecting a selectable.
 
         Supports half-close notification, isRead should be boolean indicating
         whether error resulted from doRead().
@@ -258,7 +262,8 @@ class PosixReactorBase(ReactorBase):
             selectable.connectionLost(failure.Failure(why))
 
     def installWaker(self):
-        """Install a `waker' to allow threads and signals to wake up the IO thread.
+        """
+        Install a `waker' to allow threads and signals to wake up the IO thread.
 
         We use the self-pipe trick (http://cr.yp.to/docs/selfpipe.html) to wake
         the reactor. On Windows we use a pair of sockets.
@@ -270,84 +275,6 @@ class PosixReactorBase(ReactorBase):
 
     # IReactorProcess
 
-    def _checkProcessArgs(self, args, env):
-        """
-        Check for valid arguments and environment to spawnProcess.
-
-        @return: A two element tuple giving values to use when creating the
-        process.  The first element of the tuple is a C{list} of C{str}
-        giving the values for argv of the child process.  The second element
-        of the tuple is either C{None} if C{env} was C{None} or a C{dict}
-        mapping C{str} environment keys to C{str} environment values.
-        """
-        # Any unicode string which Python would successfully implicitly
-        # encode to a byte string would have worked before these explicit
-        # checks were added.  Anything which would have failed with a
-        # UnicodeEncodeError during that implicit encoding step would have
-        # raised an exception in the child process and that would have been
-        # a pain in the butt to debug.
-        #
-        # So, we will explicitly attempt the same encoding which Python
-        # would implicitly do later.  If it fails, we will report an error
-        # without ever spawning a child process.  If it succeeds, we'll save
-        # the result so that Python doesn't need to do it implicitly later.
-        #
-        # For any unicode which we can actually encode, we'll also issue a
-        # deprecation warning, because no one should be passing unicode here
-        # anyway.
-        #
-        # -exarkun
-        defaultEncoding = sys.getdefaultencoding()
-
-        # Common check function
-        def argChecker(arg):
-            """
-            Return either a str or None.  If the given value is not
-            allowable for some reason, None is returned.  Otherwise, a
-            possibly different object which should be used in place of arg
-            is returned.  This forces unicode encoding to happen now, rather
-            than implicitly later.
-            """
-            if isinstance(arg, unicode):
-                try:
-                    arg = arg.encode(defaultEncoding)
-                except UnicodeEncodeError:
-                    return None
-                warnings.warn(
-                    "Argument strings and environment keys/values passed to "
-                    "reactor.spawnProcess should be str, not unicode.",
-                    category=DeprecationWarning,
-                    stacklevel=4)
-            if isinstance(arg, str) and '\0' not in arg:
-                return arg
-            return None
-
-        # Make a few tests to check input validity
-        if not isinstance(args, (tuple, list)):
-            raise TypeError("Arguments must be a tuple or list")
-        
-        outputArgs = []
-        for arg in args:
-            arg = argChecker(arg)
-            if arg is None:
-                raise TypeError("Arguments contain a non-string value")
-            else:
-                outputArgs.append(arg)
-
-        outputEnv = None
-        if env is not None:
-            outputEnv = {}
-            for key, val in env.iteritems():
-                key = argChecker(key)
-                if key is None:
-                    raise TypeError("Environment contains a non-string key")
-                val = argChecker(val)
-                if val is None:
-                    raise TypeError("Environment contains a non-string value")
-                outputEnv[key] = val
-        return outputArgs, outputEnv
-
-
     def spawnProcess(self, processProtocol, executable, args=(),
                      env={}, path=None,
                      uid=None, gid=None, usePTY=0, childFDs=None):
