@@ -1,10 +1,20 @@
+# Copyright (c) 2001-2008 Twisted Matrix Laboratories.
+# See LICENSE for details.
+
+"""
+Test cases covering L{twisted.python.filepath} and L{twisted.python.zippath}.
+"""
 
 import os, time, pickle, errno, zipfile
 
+from twisted.python.win32 import WindowsError, ERROR_DIRECTORY
 from twisted.python import filepath
+from twisted.python.zippath import ZipArchive
 from twisted.python.runtime import platform
+
 from twisted.trial import unittest
 
+
 class AbstractFilePathTestCase(unittest.TestCase):
 
     f1content = "file 1"
@@ -98,6 +108,130 @@ class AbstractFilePathTestCase(unittest.TestCase):
         self.failUnlessEqual(f2.open().read(), self.f2content)
 
 
+    def test_dictionaryKeys(self):
+        """
+        Verify that path instances are usable as dictionary keys.
+        """
+        f1 = self.path.child('file1')
+        f1prime = self.path.child('file1')
+        f2 = self.path.child('file2')
+        dictoid = {}
+        dictoid[f1] = 3
+        dictoid[f1prime] = 4
+        self.assertEquals(dictoid[f1], 4)
+        self.assertEquals(dictoid.keys(), [f1])
+        self.assertIdentical(dictoid.keys()[0], f1)
+        self.assertNotIdentical(dictoid.keys()[0], f1prime) # sanity check
+        dictoid[f2] = 5
+        self.assertEquals(dictoid[f2], 5)
+        self.assertEquals(len(dictoid), 2)
+
+
+    def test_dictionaryKeyWithString(self):
+        """
+        Verify that path instances are usable as dictionary keys which do not clash
+        with their string counterparts.
+        """
+        f1 = self.path.child('file1')
+        dictoid = {f1: 'hello'}
+        dictoid[f1.path] = 'goodbye'
+        self.assertEquals(len(dictoid), 2)
+
+
+    def test_childrenNonexistentError(self):
+        """
+        Verify that children raises the appropriate exception for non-existent
+        directories.
+        """
+        self.assertRaises(filepath.UnlistableError,
+                          self.path.child('not real').children)
+
+    def test_childrenNotDirectoryError(self):
+        """
+        Verify that listdir raises the appropriate exception for attempting to list
+        a file rather than a directory.
+        """
+        self.assertRaises(filepath.UnlistableError,
+                          self.path.child('file1').children)
+
+
+    def test_newTimesAreFloats(self):
+        """
+        Verify that all times returned from the various new time functions are ints
+        (and hopefully therefore 'high precision').
+        """
+        for p in self.path, self.path.child('file1'):
+            self.failUnlessEqual(type(p.getAccessTime()), float)
+            self.failUnlessEqual(type(p.getModificationTime()), float)
+            self.failUnlessEqual(type(p.getStatusChangeTime()), float)
+
+
+    def test_oldTimesAreInts(self):
+        """
+        Verify that all times returned from the various time functions are
+        integers, for compatibility.
+        """
+        for p in self.path, self.path.child('file1'):
+            self.failUnlessEqual(type(p.getatime()), int)
+            self.failUnlessEqual(type(p.getmtime()), int)
+            self.failUnlessEqual(type(p.getctime()), int)
+
+
+
+class FakeWindowsPath(filepath.FilePath):
+    """
+    A test version of FilePath which overrides listdir to raise L{WindowsError}.
+    """
+
+    def listdir(self):
+        """
+        @raise WindowsError: always.
+        """
+        raise WindowsError(
+            ERROR_DIRECTORY,
+            "A directory's validness was called into question")
+
+
+class ListingCompatibilityTests(unittest.TestCase):
+    """
+    These tests verify compatibility with legacy behavior of directory listing.
+    """
+
+    def test_windowsErrorExcept(self):
+        """
+        Verify that when a WindowsError is raised from listdir, catching
+        WindowsError works.
+        """
+        fwp = FakeWindowsPath(self.mktemp())
+        self.assertRaises(filepath.UnlistableError, fwp.children)
+        self.assertRaises(WindowsError, fwp.children)
+
+
+    def test_alwaysCatchOSError(self):
+        """
+        Verify that in the normal case where a directory does not exist, we will
+        get an OSError.
+        """
+        fp = filepath.FilePath(self.mktemp())
+        self.assertRaises(OSError, fp.children)
+
+
+    def test_keepOriginalAttributes(self):
+        """
+        Verify that the Unlistable exception raised will preserve the attributes of
+        the previously-raised exception.
+        """
+        fp = filepath.FilePath(self.mktemp())
+        ose = self.assertRaises(OSError, fp.children)
+        d1 = ose.__dict__.keys()
+        d1.remove('originalException')
+        d2 = ose.originalException.__dict__.keys()
+        d1.sort()
+        d2.sort()
+        self.assertEquals(d1, d2)
+
+
+
 def zipit(dirname, zfname):
     """
     create a zipfile on zfname, containing the contents of dirname'
@@ -112,8 +246,6 @@ def zipit(dirname, zfname):
             zf.write(fspath, arcpath)
     zf.close()
 
-from twisted.python.zippath import ZipArchive
-
 class ZipFilePathTestCase(AbstractFilePathTestCase):
 
     def setUp(self):
@@ -277,8 +409,12 @@ class FilePathTestCase(AbstractFilePathTestCase):
         oldPaths.sort()
         self.assertEquals(newPaths, oldPaths)
 
-    def testCrossMountMoveTo(self):
+
+    def test_crossMountMoveTo(self):
         """
+        C{moveTo} should be able to handle C{EXDEV} error raised by
+        C{os.rename} when trying to move a file on a different mounted
+        filesystem.
         """
         # Bit of a whitebox test - force os.rename, which moveTo tries
         # before falling back to a slower method, to fail, forcing moveTo to
@@ -301,6 +437,7 @@ class FilePathTestCase(AbstractFilePathTestCase):
         finally:
             os.rename = originalRename
 
+
     def testOpen(self):
         # Opening a file for reading when it does not already exist is an error
         nonexistent = self.path.child('nonexistent')
@@ -390,6 +527,19 @@ class FilePathTestCase(AbstractFilePathTestCase):
         self.assertRaises((OSError, IOError), existent.open)
 
 
+    def test_existsCache(self):
+        """
+        Check that C{filepath.FilePath.exists} correctly restat the object if
+        an operation has occurred in the mean time.
+        """
+        fp = filepath.FilePath(self.mktemp())
+        self.assertEquals(fp.exists(), False)
+
+        fp.makedirs()
+        self.assertEquals(fp.exists(), True)
+
+
+
 from twisted.python import urlpath
 
 class URLPathTestCase(unittest.TestCase):
@@ -405,7 +555,7 @@ class URLPathTestCase(unittest.TestCase):
 
     def testSiblingString(self):
         self.assertEquals(str(self.path.sibling('baz')), 'http://example.com/foo/baz')
-        
+
         # The sibling of http://example.com/foo/bar/
         #     is http://example.comf/foo/bar/baz
         # because really we are constructing a sibling of
